function [t,xt] = lorenz63(x_0,tmax,param_out)
 % [t,xt] = lorenz3v(x0,tmax)
 % This program contains the evolution of the Lorenz 1963 3-variable model
 % Inputs:  - x_0, a vector of initial positions
 %          - tmax, the maximum time
 %          - the parameters for the model, if empty it uses the default
 % Outputs: - t, the time array
 %          - xt, the nature run
 
 global N param
 
 % The model can be ran as perfect or imperfect by changing the following switch:
 % Is the model perfect or not?
 perf = 'yes';
 
 if isempty(param_out)
  param = [10,8/3,28];
 elseif ~isempty(param_out)
  param = param_out;
 end
 
 % the time array
 tstep = 0.01;
 t = (0:tstep:tmax)';    

 % Note that with this step size, 8 steps are approximately linear windows,
 % 25 steps are typical nonlinear windows
 
 % Now, define the vectors for space vars. They're organized in a matrix of 3 columns [x,y,z]
 N = 3;
 xt = NaN(length(t),N);
 xt(1,:) = x_0;
  
 % The cycle containing time integration
 for i = 1:length(t)-1 % (For each time step)
  Varsold = xt(i,:);
  % we integrate using Runge-Kutta-4
  Varsnew = intfwd(Varsold,tstep,perf);
  xt(i+1,:)= Varsnew;
  % i
 end
end


%% Integration functions
function [Varsnew,betaout] = intfwd(Varsold,tstep,perf)
 global N
 
 % Perfect model uses Runge Kutta 4
 if strcmp(perf,'yes')
  k1 = f(Varsold);
  k2 = f(Varsold+1/2.*tstep.*k1);
  k3 = f(Varsold+1/2.*tstep.*k2);
  k4 = f(Varsold+tstep.*k3);
  Varsnew = Varsold + 1/6*tstep*(k1+2*k2+2*k3+k4); 
  betaout = NaN;
 % Imperfect model uses Euler-Maruyama
 elseif strcmp(perf,'no')
  % define model error     
  modelstd = 2;
  Q = modelstd^2* eye(N,N);
  beta = Q*randn(N,1);
  dW = sqrt(tstep)*beta'; 
  Varsnew = Varsold + tstep*f(Varsold) + dW;
 end
end


%% This function contains the actual L63 model
function k = f(x)
 global param
 % the parameters
 sigma = param(1,1);     b = param(1,2);    r = param(1,3);
 % initialize 
 k = NaN(size(x));
 % the Lorenz equations
 k(1,1) = sigma*(x(1,2)-x(1,1));
 k(1,2) = (x(1,1)*(r-x(1,3))-x(1,2));
 k(1,3) = x(1,1)*x(1,2)-b*x(1,3);
end


